-- (c) M.P.I 98/99 -- Date: 98.12.03

module BTree where

import Mpi
import List

-- (1) Datatype definition -----------------------------------------------------

data BTree a = Empty | Node(a, (BTree a, BTree a)) deriving Show

inBTree = either (const Empty) Node

outBTree :: BTree a -> Either () (a,(BTree a,BTree a))
outBTree Empty              = Left ()
outBTree (Node (a,(t1,t2))) = Right(a,(t1,t2))

-- (2) Ana + cata + hylo -------------------------------------------------------

cata a = a . (rec (cata a)) . outBTree

ana f = inBTree . (rec (ana f) ) . f

hylo a c = cata a . ana c

rec f = id -|- (id >< (f >< f))

-- (3) Map ---------------------------------------------------------------------

instance Functor BTree
         where map f = cata ( inBTree . (id -|- (f >< (id >< id))) )

-- (4) Examples ----------------------------------------------------------------

-- (4.1) Quicksort -------------------------------------------------------------

qSort :: Ord a => [a] -> [a]
qSort = hylo (either (const []) join) sep

-- where

join(x,(l,r))=l++[x]++r

sep []    = Left ()
sep (h:t) = Right (h,(s,l)) where (s,l) = part (<h) t

part p []                = ([],[])
part p (h:t) | p h       = let (s,l) = part p t in (h:s,l)
             | otherwise = let (s,l) = part p t in (s,h:l)

-- (4.2) Traces ----------------------------------------------------------------

traces :: Eq a => BTree a -> [[a]]
traces = cata (either (const [[]]) put)

-- where

put(a,(l,r)) = union (map (a:) l) (map (a:) r) 

--------------------------------------------------------------------------------

