-- (c) M.P.I 98/99 -- Date: 98.12.03

module LTree where

import Mpi

-- (1) Datatype definition -----------------------------------------------------

data LTree a = Leaf a | Split (LTree a, LTree a) deriving Show

inLTree = either Leaf Split

outLTree :: LTree a -> Either a (LTree a,LTree a)
outLTree (Leaf a)     = Left a
outLTree (Split (t1,t2)) = Right (t1,t2)

-- (2) Ana + cata + hylo -------------------------------------------------------

cata a = a . (rec (cata a)) . outLTree

ana f = inLTree . (rec (ana f) ) . f

hylo a c = cata a . ana c

rec f = id -|- (f >< f)

-- (3) Map ---------------------------------------------------------------------

instance Functor LTree
         where map f = cata ( inLTree . ( f -|- (id >< id)))

-- (4) Examples ----------------------------------------------------------------

-- (4.1) Flatten ---------------------------------------------------------------

flatten = cata (either sing conc)

-- where

sing   x = [x]
conc(l,r)= l ++ r

-- (4.2) Double factorial ------------------------------------------------------

fac 0 = 1
fac n = (hylo (either id mul) interval) (1,n)

-- where

mul(n,m)=n*m

interval(n,m) | n==m      = Left n
              | otherwise = Right((n,k),(k+1,m))
                            where k = div (n+m) 2

-- (4.3) Fibonacci -------------------------------------------------------------

fib =  hylo (either id add) getN

-- where

add(x,y)=x+y

getN n | n < 2     = Left 1
       | otherwise = Right(n-1,n-2)

-- (4.4) Merge sort ------------------------------------------------------------

mSort :: Ord a => [a] -> [a]
mSort [] = []
mSort l = hylo (either sing merge) select l

--where

merge (l,[])                  = l
merge ([],r)                  = r
merge (x:xs,y:ys) | x < y     = x : merge(xs,y:ys) 
                  | otherwise = y : merge(x:xs,ys)

select [x] = Left x
select l   = Right (sep l)

sep []    = ([],[])
sep (x:t) = let (l,r) = sep t in (x:r,l)  -- a List cata

--------------------------------------------------------------------------------

